#include "Game.h"

#include "CubeModel.h"
#include "ShipModel.h"

/* Our Game class is going to take responsibility for the game code
 * 
 *
 *
 *
 **/

Game::Game()
{
}

Game::~Game()
{
	
} 


bool Game::Init(Graphics* Gr)
{
	
	MyGraphics = Gr;
	MyFiles* Handler = new MyFiles();

	// as an example lets make 2 objects, an old Cube and a new Ship, both are derived from ObjectModel and we have a vector of ObjectModel pointers
	// where we plan to store them, if we add more types of object which inherit ObjectModel we can simply add them to the list, and let thier, own 
	// update and draw functions handline them
	
	ObjectModel* T2;   // so both types even though quite different use the same base to create them
	
	T2 = new CubeModel(Handler); // make a new cube
	glm::vec3 Pos = glm::vec3(10.0f, 0.0f, 0.0f); // set a position
	T2->Scales = glm::vec3(0.1f, 0.1f, 0.1f);		 // a cube is actually quite large (screen size) so shrink it down
	T2->SetPosition(Pos); 
	MyObjects.push_back(T2); // store in the vector ready for the game loop to process
	T2->StoreGraphicClass(Gr); // make sure it knows the where the graphics class is, (for now it contains our attribute/uniform info)
	Gr->Init(T2); // set it up

// repeat for a different kind of model	this time a full Obj model
	T2 = new ShipModel((char*)"../Resources/Models/brian_01.obj", Handler);
	Pos = glm::vec3(-8.0f, 0.0f, 0.0f);
	T2->Scales = glm::vec3(0.8f, 0.8f, 0.8f);		 
	T2->SetPosition(Pos);
	T2->StoreGraphicClass(Gr);
	Gr->Init(T2);
	MyObjects.push_back(T2);
	
	
}

bool Game::Update()
{

	
// we're going to keep track of time so need some structures to hold the time in linux's strange format
	struct timeval t1, t2;
	struct timezone tz;
	float deltatime;
	float totaltime = 0.0f;
	unsigned int frames = 0;

	gettimeofday(&t1, &tz);
	int Counter = 0;
	while (Counter++ < 2000)
	{
		

		gettimeofday(&t2, &tz);
		deltatime = (float)(t2.tv_sec - t1.tv_sec + (t2.tv_usec - t1.tv_usec) * 0.0000001f);
		t1 = t2;
	
		// Clear the color buffer
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		for (int i = 0; i < MyObjects.size(); i++)
		{
			MyObjects[i]->Update(); // even though we have a cube model, and a shipmodel, 2 different types, they inherit virtual functions which have been supplied
			MyObjects[i]->Draw(); // so they will process the update and draw functions provided by each class.
		} // for
		
		// after our draw we need to swap buffers to display on screen
		eglSwapBuffers(MyGraphics->state.display, MyGraphics->state.surface);
		
// now work out how long that took
		totaltime += deltatime;
		frames++;
		if (totaltime >  1.0f)
		{
			printf("%4d frames rendered in %1.4f seconds -> FPS=%3.4f\n", frames, totaltime, frames / totaltime);
			totaltime -= 1.0f;
			frames = 0;
		}
	}
	
	

}
