
#include "../Headers/Graphics.h"

using namespace glm;

// moving initialisaiton and processing to here


Graphics::Graphics()
{
	// don't really want to do anything here but could move init here
}
Graphics::~Graphics()
{
	
}


void Graphics::esInitContext()
{
	if (p_state != NULL)
	{
		memset(p_state, 0, sizeof(Target_State));
	}
}

/*
Set up the EGL and in the case of Raspberry DispamX
Set up OpenGL states needed
*/
void Graphics::init_ogl( int width, int height)
{
	int32_t success = 0;
	EGLBoolean result;
	EGLint num_config;

	bcm_host_init(); //RPI needs this
	

	
	
//RPI setup is a little different to normal EGL
	DISPMANX_ELEMENT_HANDLE_T DispmanElementH;
	DISPMANX_DISPLAY_HANDLE_T DispmanDisplayH;
	DISPMANX_UPDATE_HANDLE_T DispmanUpdateH;
	VC_RECT_T dest_rect;
	VC_RECT_T src_rect;
	EGLConfig config;

// get an EGL display connection
	state.display = eglGetDisplay(EGL_DEFAULT_DISPLAY);

// initialize the EGL display connection
	result = eglInitialize(state.display, NULL, NULL);

// get an appropriate EGL frame buffer configuration
	result = eglChooseConfig(state.display, attribute_list, &config, 1, &num_config);
	assert(EGL_FALSE != result);

// get an appropriate EGL frame buffer configuration
	result = eglBindAPI(EGL_OPENGL_ES_API);
	assert(EGL_FALSE != result);

// create an EGL rendering context
	state.context = eglCreateContext(state.display, config, EGL_NO_CONTEXT, context_attributes);
	assert(state.context != EGL_NO_CONTEXT);

// create an EGL window surface

	state.width = width;
	state.height = height;

	dest_rect.x = 0;
	dest_rect.y = 0;
	dest_rect.width = state.width; // it needs to know our window size
	dest_rect.height = state.height;

	src_rect.x = 0;
	src_rect.y = 0;

	DispmanDisplayH = vc_dispmanx_display_open(0);
	DispmanUpdateH = vc_dispmanx_update_start(0);

	DispmanElementH = 	vc_dispmanx_element_add(
		DispmanUpdateH,
		DispmanDisplayH,
		0/*layer*/,
		&dest_rect,
		0/*source*/,
		&src_rect,
		DISPMANX_PROTECTION_NONE, 
		0 /*alpha value*/,
		0/*clamp*/,
		(DISPMANX_TRANSFORM_T) 0/*transform*/);

	state.nativewindow.element = DispmanElementH;
	state.nativewindow.width = state.width;
	state.nativewindow.height = state.height;
	vc_dispmanx_update_submit_sync(DispmanUpdateH);

	state.surface = eglCreateWindowSurface(state.display, config, &(state.nativewindow), NULL);
	assert(state.surface != EGL_NO_SURFACE);

	    // connect the context to the surface
	result = eglMakeCurrent(state.display, state.surface, state.surface, state.context);
	assert(EGL_FALSE != result);

// Some OpenGLES2.0 states that we might need
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	
	// Setup the viewport
	glViewport(0, 0, width, height);
}



/*
 Now we have be able to create a shader object, pass the shader source
 and them compile the shader.
*/
GLuint Graphics::LoadShader(GLenum type, const char *shaderSrc)
{	
// 1st create the shader object
	GLuint TheShader = glCreateShader(type);
	
	if (TheShader == 0) return FALSE; // can't allocate so stop.
	
// pass the shader source then compile it
	glShaderSource(TheShader, 1, &shaderSrc, NULL);
	glCompileShader(TheShader);
	
	GLint  IsItCompiled;
	
// After the compile we need to check the status and report any errors
	glGetShaderiv(TheShader, GL_COMPILE_STATUS, &IsItCompiled);
	if (!IsItCompiled)
	{
		GLint RetinfoLen = 0;
		glGetShaderiv(TheShader, GL_INFO_LOG_LENGTH, &RetinfoLen);
		if (RetinfoLen > 1)
		{ // standard output for errors
			char* infoLog = (char*) malloc(sizeof(char) * RetinfoLen);
			glGetShaderInfoLog(TheShader, RetinfoLen, NULL, infoLog);
			fprintf(stderr, "Error compiling this shader:\n%s\n", infoLog);
			free(infoLog);
		}
		glDeleteShader(TheShader);
		return FALSE;
	}
	return TheShader;
}



/*
Initialise an object so that it uses the correct values

Now, for the moment since all our models use the same shader this is ok
but soon we will use different shaders for different models, and this won't
be as useful
*/

int Graphics::Init(ObjectModel* TheModel)
{
	
	// hard code for now but we have a shader manager ready to take care of multiuple shaders

		GLbyte vShaderStr[] =
		"precision mediump float;                            \n"
		"attribute vec3 a_position;   \n"
		"attribute vec2 a_texCoord;   \n"
		"uniform mat4 MVP;            \n"
		"varying vec2 v_texCoord;     \n"
		"void main()                  \n"
		"{ 							  \n"
	
		" gl_Position =  MVP * vec4(a_position,1);\n"
		" v_texCoord = a_texCoord;  \n"
		"}                           \n";

	GLbyte fShaderStr[] =
		"precision mediump float;                            \n"
		"varying vec2 v_texCoord;                            \n"
		"uniform sampler2D s_texture;                        \n"
		"uniform vec4  Ambient;\n"
		"void main()                                         \n"
		"{                                                   \n"
		"  gl_FragColor = texture2D( s_texture, v_texCoord )*Ambient;\n"
		"}\n";
		
	
 // Load and compile the vertex/fragment shaders
	TheModel->vertexShader = LoadShader(GL_VERTEX_SHADER, (const char*)vShaderStr);
	TheModel->fragmentShader = LoadShader(GL_FRAGMENT_SHADER, (const char*)fShaderStr);
	
// Create the program object	
	TheModel->programObject = glCreateProgram();
	if (TheModel->programObject == 0) 	return 0;
	
// now we have the V and F shaders  attach them to the progam object
	glAttachShader(TheModel->programObject, TheModel->vertexShader);
	glAttachShader(TheModel->programObject, TheModel->fragmentShader);
	
// Link the program
	glLinkProgram(TheModel->programObject);
// Check the link status
	GLint AreTheylinked;
	glGetProgramiv(TheModel->programObject, GL_LINK_STATUS, &AreTheylinked);
	if (!AreTheylinked)
	{
		GLint RetinfoLen = 0;
// check and report any errors
		glGetProgramiv(TheModel->programObject, GL_INFO_LOG_LENGTH, &RetinfoLen);
		if (RetinfoLen > 1)
		{
			GLchar* infoLog = (GLchar*)malloc(sizeof(char) * RetinfoLen);
			glGetProgramInfoLog(TheModel->programObject, RetinfoLen, NULL, infoLog);
			fprintf(stderr, "Error linking program:\n%s\n", infoLog);
			free(infoLog);
		}
		glDeleteProgram(TheModel->programObject);
		return FALSE;
	}

	// Get the attribute locations
	TheModel->positionLoc = glGetAttribLocation(TheModel->programObject, "a_position");
	TheModel->texCoordLoc = glGetAttribLocation(TheModel->programObject, "a_texCoord");

	// Get the sampler location
	TheModel->samplerLoc = glGetUniformLocation(TheModel->programObject, "s_texture");
	
	if (glGetError() == GL_NO_ERROR)	return TRUE;
	else 
	printf("Oh bugger, models graphic init failed\n");
	return FALSE;

}



///
// Create a simple width x height texture image with four different colors
//
GLuint Graphics::CreateSimpleTexture2D(int width, int height, char* TheData)
{
	// Texture object handle
	GLuint textureId;

	// Use tightly packed data
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	// Generate a texture object
	glGenTextures(1, &textureId);

	// Bind the texture object
	glBindTexture(GL_TEXTURE_2D, textureId);

	// Load the texture

	glTexImage2D(GL_TEXTURE_2D,
		0,
		GL_RGBA,
		width,
		height,
		0,
		GL_RGBA,
		GL_UNSIGNED_BYTE,
		TheData);
	
	if (glGetError() != GL_NO_ERROR) printf("Oh bugger CreateSimeTexture2D failed!");

	// Set the filtering mode
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);


	
	if (glGetError() == GL_NO_ERROR)	return textureId;
	printf("Oh bugger something went really wrong");

	return textureId;
}


