/* Model Manager
Primary function is to load models and their mat/textures and store them in OAB's or in ObjModel data
Most OBJ's will go to the OAB and use TinyObjLoader to do their convertion and storage.
Contains the maps and texture lists to prevent double loads of texture files and OAB's

*/


/*Parts of this file, specifically the routines, LoadandConvert and CalcNormal as well as the data structures used by them
 have been converted and modifed from Soyo Fujita's viewer example code in TinyOBJLoader which was distributed under the MIT 
 license. In accordance with that license I acknowledge the original copyright and ownership of those routines and reprint 
 the copyright notice here:

The MIT License (MIT)

Copyright (c) 2012-2017 Syoyo Fujita and many contributors.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "../Headers/ModelManager.h"





ModelManager::ModelManager() 
{
//constructor set ups here

};

ModelManager::~ModelManager()
{
// make sure the vectors are closed and data systems shut down

}


bool ModelManager::LoadModel(ObjectModel* MyModel, MyFiles* FH,char* filename)
{
	LoadandConvert(&MyModel->gdrawObjects, MyModel->materials, Totaltextures, filename, FH) ;

}

bool  ModelManager::LoadandConvert(
									std::vector<ObjectModel::DrawObject>* drawObjects,
									std::vector<tinyobj::material_t>& materials,
									std::map<std::string,GLuint>& textures,
									const char* filename,
									MyFiles* FH)
	 {
// create a few local working variables
	tinyobj::attrib_t attrib;
	std::vector<tinyobj::shape_t> shapes;
	std::string err;
// Do the actual load
	bool ret =    tinyobj::LoadObj(&attrib, &shapes, &materials, &err, filename, "../Resources/Models/");

// check for errors	
	if (!err.empty()) 
	{
		char *cstr = &err[0u];// printf needs chars
		printf("oh Bugger can't load this %s.obj file returns error %s \n", filename, cstr);	
		return false; // we failed no sense in continuing.
	}
//it may have loaded but not parsed correctly	
	if (!ret) 
	{
		printf("The %s.obj seems to be corrupt or incorrectly formated\n", filename);
		return false;
	}
		
	
// output the use data for reference
	printf("Attrib data for %s is :-\n", filename);
	printf("# vertices  : %d\n", (int)(attrib.vertices.size()) / 3);
	printf("# normals   : %d\n", (int)(attrib.normals.size()) / 3);
	printf("# texcoords : %d\n", (int)(attrib.texcoords.size()) / 2);
	printf("Other data  :-\n", filename);

	printf("# materials : %d\n", (int)materials.size());
	printf("# shapes    : %d\n", (int)shapes.size());

	  // Append `default` material
	materials.push_back(tinyobj::material_t());

	char* base_dir = (char*) "../Resources/Models/";
	// Parse the materials and Load and create textures
	{
		
		for (size_t m = 0; m < materials.size(); m++) {
			tinyobj::material_t* mp = &materials[m];
          
			if (mp->diffuse_texname.length() > 0) {
			    // Only load the texture if it is not already loaded
				if (textures.find(mp->diffuse_texname) == textures.end()) {
					GLuint texture_id;
					int w, h;
					
					std::string texture_filename = mp->diffuse_texname;
					texture_filename = base_dir + mp->diffuse_texname;
					
				
					char *cstr = &texture_filename[0u]; // embarrasing side effect of the decision to use char.....
					char* image = FH->Load(cstr, &w, &h); 

					if (!image) {
						printf("Unable to load texture:%s \n", cstr);
						exit(1);
					}
				
					printf("Texture info for %s :-\n", cstr, filename);

					printf("# width   : %d\n", w);
					printf("# Height  : %d\n", h);
					printf("# comp    : %d\n", FH->comp);


					glGenTextures(1, &texture_id);
					glBindTexture(GL_TEXTURE_2D, texture_id);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
					
					
					if (FH->comp == 4)	// check if we had an alpha texture or a plane (strictly speaking any value not 3 is wrong)	
					{
						glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
					}
					else
					{
						if (FH->comp != 3) printf("Made a default non alpha texture, comp was :%d\n", FH->comp);
						glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);	
					}
					
					if (glGetError() != GL_NO_ERROR) 
					{
						printf("Oh bugger:- Model texture creation error, but attempting to carry on.\n");
						
					}

					glBindTexture(GL_TEXTURE_2D, 0);
					free(image); // release the cpu memory once its been put in the GPU
					textures.insert(std::make_pair(mp->diffuse_texname, texture_id));

				}
			}
		}
	}
	
	
	{
		for (size_t s = 0; s < shapes.size(); s++) {
			ObjectModel::DrawObject o;
			std::vector<float> vb;  // pos(3float), normal(3float), color(3float)
			for (size_t f = 0; f < shapes[s].mesh.indices.size() / 3; f++) {
				tinyobj::index_t idx0 = shapes[s].mesh.indices[3 * f + 0];
				tinyobj::index_t idx1 = shapes[s].mesh.indices[3 * f + 1];
				tinyobj::index_t idx2 = shapes[s].mesh.indices[3 * f + 2];
        
				int current_material_id = shapes[s].mesh.material_ids[f];

				if ((current_material_id < 0) || (current_material_id >= static_cast<int>(materials.size()))) {
				  // Invaid material ID. Use default material.
					current_material_id = materials.size() - 1; // Default material is added to the last item in `materials`.
				}
				//if (current_material_id >= materials.size()) {
				//    std::cerr << "Invalid material index: " << current_material_id << std::endl;
				//}
				//
				float diffuse[3];
				for (size_t i = 0; i < 3; i++) {
					diffuse[i] = materials[current_material_id].diffuse[i];
				}

				float tc[3][2];
				if (attrib.texcoords.size() > 0) {
					assert(attrib.texcoords.size() > 2 * idx0.texcoord_index + 1);
					assert(attrib.texcoords.size() > 2 * idx1.texcoord_index + 1);
					assert(attrib.texcoords.size() > 2 * idx2.texcoord_index + 1);


					tc[0][0] = attrib.texcoords[2 * idx0.texcoord_index];

					tc[0][1] = 1.0f - attrib.texcoords[2 * idx0.texcoord_index + 1];
					tc[1][0] = attrib.texcoords[2 * idx1.texcoord_index];
					tc[1][1] = 1.0f - attrib.texcoords[2 * idx1.texcoord_index + 1];
					tc[2][0] = attrib.texcoords[2 * idx2.texcoord_index];
					tc[2][1] = 1.0f - attrib.texcoords[2 * idx2.texcoord_index + 1];
					
/*
				float tc[3][2];
				if (attrib.texcoords.size() > 0) {
					assert(attrib.texcoords.size() > 2 * idx0.texcoord_index + 1);
					assert(attrib.texcoords.size() > 2 * idx1.texcoord_index + 1);
					assert(attrib.texcoords.size() > 2 * idx2.texcoord_index + 1);
					tc[0][0] = attrib.texcoords[2 * idx0.texcoord_index] / materials[current_material_id].diffuse_texopt.scale[0];
					tc[0][1] = 1.0f - attrib.texcoords[2 * idx0.texcoord_index + 1] / materials[current_material_id].diffuse_texopt.scale[1];
					tc[1][0] = attrib.texcoords[2 * idx1.texcoord_index] / materials[current_material_id].diffuse_texopt.scale[0];
					tc[1][1] = 1.0f - attrib.texcoords[2 * idx1.texcoord_index + 1] / materials[current_material_id].diffuse_texopt.scale[1];
					tc[2][0] = attrib.texcoords[2 * idx2.texcoord_index] / materials[current_material_id].diffuse_texopt.scale[0];
					tc[2][1] = 1.0f - attrib.texcoords[2 * idx2.texcoord_index + 1] / materials[current_material_id].diffuse_texopt.scale[1];
*/
				}
				else {
					tc[0][0] = 0.0f;
					tc[0][1] = 0.0f;
					tc[1][0] = 0.0f;
					tc[1][1] = 0.0f;
					tc[2][0] = 0.0f;
					tc[2][1] = 0.0f;
				}

				float v[3][3];
				for (int k = 0; k < 3; k++) {
					int f0 = idx0.vertex_index;
					int f1 = idx1.vertex_index;
					int f2 = idx2.vertex_index;
					assert(f0 >= 0);
					assert(f1 >= 0);
					assert(f2 >= 0);

					v[0][k] = attrib.vertices[3 * f0 + k];
					v[1][k] = attrib.vertices[3 * f1 + k];
					v[2][k] = attrib.vertices[3 * f2 + k];
					
				//	bmin[k] = std::min(v[0][k], bmin[k]);
				//	bmin[k] = std::min(v[1][k], bmin[k]);
				//	bmin[k] = std::min(v[2][k], bmin[k]);
				//	bmax[k] = std::max(v[0][k], bmax[k]);
				//	bmax[k] = std::max(v[1][k], bmax[k]);
				//	bmax[k] = std::max(v[2][k], bmax[k]);
				}

				float n[3][3];
				if (attrib.normals.size() > 0) {
					int f0 = idx0.normal_index;
					int f1 = idx1.normal_index;
					int f2 = idx2.normal_index;
					assert(f0 >= 0);
					assert(f1 >= 0);
					assert(f2 >= 0);
					for (int k = 0; k < 3; k++) {
						n[0][k] = attrib.normals[3 * f0 + k];
						n[1][k] = attrib.normals[3 * f1 + k];
						n[2][k] = attrib.normals[3 * f2 + k];
					}
				}
				else {
				  // compute geometric normal
					CalcNormal(n[0], v[0], v[1], v[2]);
					n[1][0] = n[0][0];
					n[1][1] = n[0][1];
					n[1][2] = n[0][2];
					n[2][0] = n[0][0];
					n[2][1] = n[0][1];
					n[2][2] = n[0][2];
				}

				for (int k = 0; k < 3; k++) {
					vb.push_back(v[k][0]);
					vb.push_back(v[k][1]);
					vb.push_back(v[k][2]);
					
				//	printf("v1 v2 v3 =%f  %f  %f\n", v[k][0], v[k][1], v[k][2]);
					
					vb.push_back(n[k][0]);
					vb.push_back(n[k][1]);
					vb.push_back(n[k][2]);
					// Combine normal and diffuse to get color.
					float normal_factor = 0.2;
					float diffuse_factor = 1 - normal_factor;
					float c[3] = {
						n[k][0] * normal_factor + diffuse[0] * diffuse_factor,
						n[k][1] * normal_factor + diffuse[1] * diffuse_factor,
						n[k][2] * normal_factor + diffuse[2] * diffuse_factor
					};
					float len2 = c[0] * c[0] + c[1] * c[1] + c[2] * c[2];
					if (len2 > 0.0f) {
						float len = sqrtf(len2);

						c[0] /= len;
						c[1] /= len;
						c[2] /= len;
					}
					vb.push_back(c[0] * 0.5 + 0.5);
					vb.push_back(c[1] * 0.5 + 0.5);
					vb.push_back(c[2] * 0.5 + 0.5);
          
					vb.push_back(tc[k][0]);
					vb.push_back(tc[k][1]);
				}
			}

			o.vb = 0;
			o.numTriangles = 0;

			      // OpenGL viewer does not support texturing with per-face material.
			if (shapes[s].mesh.material_ids.size() > 0 && shapes[s].mesh.material_ids.size() > s) {
			    // Base case
				o.material_id = shapes[s].mesh.material_ids[s];
			}
			else {
				o.material_id = materials.size() - 1; // = ID for default material.
			}
          
				
			if (vb.size() > 0) {
				glGenBuffers(1, &o.vb);
				glBindBuffer(GL_ARRAY_BUFFER, o.vb);
				glBufferData(GL_ARRAY_BUFFER,
					vb.size() * sizeof(float),
					&vb.at(0),
					GL_STATIC_DRAW);
				
				o.numTriangles = vb.size() / (3 + 3 + 3 + 2); 
				
				printf("shape[%d] # of triangles = %d\n",
					static_cast<int>(s),
					o.numTriangles);
				
				GLsizei size;
				glGetBufferParameteriv(GL_ARRAY_BUFFER, GL_BUFFER_SIZE, &size);
				printf("VBO is reported as %d \n", size);
			}

			drawObjects->push_back(o);
		}
	}
	return true;
}
